<?php
/**
 * Dashly plugin setup
 *
 * Sets all the dependencies for plugin to work
 *
 * @package Dashly
 */

// Require file with callbacks.
require_once 'class-dashlyhooks.php';

/**
 * Class DashlyBase
 *
 * Plugin init class
 */
class DashlyBase {

	/**
	 * Plugin was initiated
	 *
	 * @var bool
	 */
	private static $initiated = false;

	/**
	 * Initializing required hooks if they not yet initiated.
	 */
	public static function init() {
		if ( ! self::$initiated ) {
			self::add_hooks();
			self::$initiated = true;
		}
	}


	/**
	 * Add hooks required for plugin to work correctly
	 */
	private static function add_hooks() {
		add_filter(
			'plugin_action_links_' . DASHLY_PLUGIN_BASE,
			array(
				'DashlyBase',
				'action_links',
			)
		); // Adding action links for plugin in admin panel.
		add_action( 'admin_menu', array( 'DashlyBase', 'add_pages' ) ); // Adding plugin options page.

		add_action( 'wp_enqueue_scripts', array('DashlyBase', 'plugin_main' ) ); // Adding Dashly widget		

		add_action( 'wp_login', array( 'DashlyHooks', 'user_login' ) );
		add_action( 'wp_footer', array( 'DashlyHooks', 'login_authentication' ) );

		add_action(
			'woocommerce_after_single_product',
			array(
				'DashlyHooks',
				'product_viewed',
			)
		); // User viewed product.
		add_action( 'woocommerce_after_cart', array( 'DashlyHooks', 'cart_viewed' ) ); // User viewed cart.

		add_action(
			'woocommerce_add_to_cart',
			array(
				'DashlyHooks',
				'product_added',
			)
		);
		add_action(
			'woocommerce_ajax_added_to_cart',
			array(
				'DashlyHooks',
				'product_added',
			)
		); // User added product to cart via ajax.

		add_action(
			'woocommerce_before_checkout_form',
			array(
				'DashlyHooks',
				'order_started',
			)
		); // User opened checkout page.
		add_action(
			'woocommerce_checkout_order_processed',
			array(
				'DashlyHooks',
				'order_completed',
			)
		); // User completed checkout and order was created.

		add_action( 'woocommerce_order_status_changed', array( 'DashlyHooks', 'order_status_changed' ), 10, 3 );
		add_action( 'woocommerce_thankyou', array( 'DashlyHooks', 'order_authentication' ) );

		register_uninstall_hook( __FILE__, array( 'dashly', 'delete_options' ) );
	}

	/**
	 *  Plugin action links.
	 *
	 * @param array $actions action links array.
	 *
	 * @return mixed
	 */
	public static function action_links( $actions ) {
		$actions[] = '<a href="plugins.php?page=dashly">' . __( 'Settings', 'dashly' ) . '</a>';

		return $actions;
	}

	/**
	 * Show plugin Options page in menu
	 */
	public static function add_pages() {
		add_submenu_page(
			'plugins.php',
			__( 'Dashly', 'dashly' ),
			__( 'Dashly', 'dashly' ),
			'manage_options',
			'dashly',
			array( 'DashlyBase', 'settings_page' )
		);
		add_action( 'admin_init', array( 'DashlyBase', 'add_options' ) ); // Adding plugins options.
	}

	/**
	 * Creating plugin options in wp
	 */
	public static function add_options() {
		if ( ! get_option( 'dashly_api_key' ) ) {
			add_option( 'dashly_api_key' );
		}
		if ( ! get_option( 'dashly_api_secret' ) ) {
			add_option( 'dashly_api_secret' );
		}
		if ( ! get_option( 'dashly_auth_key' ) ) {
			add_option( 'dashly_auth_key' );
		}
		if ( ! get_option( 'dashly_auth' ) ) {
			add_option( 'dashly_auth' );
		}
	}

	/**
	 * Delete plugin options when plugin deleted
	 */
	public static function delete_options() {
		delete_option( 'dashly_api_key' );
		delete_option( 'dashly_api_secret' );
		delete_option( 'dashly_auth_key' );
		delete_option( 'dashly_auth' );
	}


	/**
	 * Form and initialize options page
	 */
	public static function settings_page() {
		if ( isset( $_REQUEST['dashly_plugin_form_submit'] )
			 && check_admin_referer( 'dashly_plugin_settings', 'dashly_plugin_nonce' ) // Updating options if options form was submited.
		) {
			if ( isset( $_REQUEST['dashly_api_key'] ) ) {
				update_option( 'dashly_api_key', sanitize_text_field( wp_unslash( $_REQUEST['dashly_api_key'] ) ) );
			} else {
				update_option( 'dashly_api_key', '' );
			}

			if ( isset( $_REQUEST['dashly_api_secret'] ) ) {
				update_option( 'dashly_api_secret', sanitize_text_field( wp_unslash( $_REQUEST['dashly_api_secret'] ) ) );
			} else {
				update_option( 'dashly_api_secret', '' );
			}

			if ( isset( $_REQUEST['dashly_auth_key'] ) ) {
				update_option( 'dashly_auth_key', sanitize_text_field( wp_unslash( $_REQUEST['dashly_auth_key'] ) ) );
			} else {
				update_option( 'dashly_auth_key', '' );
			}

			if ( isset( $_REQUEST['dashly_auth'] ) ) {
				update_option( 'dashly_auth', sanitize_text_field( wp_unslash( $_REQUEST['dashly_auth'] ) ) );
			} else {
				update_option( 'dashly_auth', '' );
			}

			$message = __( 'Settings saved', 'dashly' ); // Everything's OK.
		}
		if ( ! isset( $message ) ) {
			$message = __( 'Failed', 'dashly' ); // Something went wrong.
		}
		$page = DASHLY_PLUGIN_DIR . 'options.php';
		ob_start();
		include $page; // Including options page template.
		echo ob_get_clean();
	}


	/**
	 * Adding Dashly script to every page if it's not admin panel
	 */
	public static function plugin_main() {										
		// Skip our script if it's admin panel
		if ( ! is_admin() ) {			   
			$settings = DashlyHooks::get_settings();
			if ( ! empty( $settings['api_key'] ) ) { // If api_key option is set add code to each non-admin page.
				// @formatter:off
				?>
				<!-- Dashly BEGIN -->
				<script type="text/javascript">
					!function(){function t(t,e){return function(){window.dashlyasync.push(t,arguments)}}if("undefined"==typeof dashly){var e=document.createElement("script");e.type="text/javascript",e.async=!0,e.src="//cdn.dashly.app/api.min.js",document.getElementsByTagName("head")[0].appendChild(e),window.dashly={},window.dashlyasync=[],dashly.settings={};for(var n=["connect","track","identify","auth","oth","onReady","addCallback","removeCallback","trackMessageInteraction"],a=0;a<n.length;a++)dashly[n[a]]=t(n[a])}}(),dashly.connect('<?php echo esc_textarea( $settings['api_key'] ); ?>');
				</script>
				<!-- Dashly END -->
				<?php
				// @formatter:on
			}
		}
	}
}
